use std::path::Path;

use calibre_db::{
    calibre::{Calibre, DataStoreError},
    data::author::Author,
};
use snafu::{ResultExt, Snafu};

use super::SortOrder;
use crate::data::book::Book;

#[derive(Debug, Snafu)]
pub enum SingleAuthorError {
    #[snafu(display("Failed to fetch author data."))]
    AuthorData { source: DataStoreError },
    #[snafu(display("Failed to fetch books from author."))]
    BookData { source: DataStoreError },
}

pub async fn single(
    id: u64,
    calibre: &Calibre,
    library_path: &Path,
) -> Result<(Author, Vec<Book>), SingleAuthorError> {
    let author = calibre.scalar_author(id).context(AuthorDataSnafu)?;
    let books = calibre
        .author_books(id, u32::MAX.into(), None, SortOrder::ASC.into())
        .context(BookDataSnafu)?;
    let books = books
        .iter()
        .filter_map(|x| Book::full_book(x, calibre, library_path))
        .collect::<Vec<Book>>();

    Ok((author, books))
}
